@@ -11,7 +11,7 @@ import tempfile
 import uuid
 import hmac
 from xpra.os_util import strtobytes, bytestostr, monotonic_time, WIN32, OSX, POSIX
-from xpra.net.crypto import get_digests, get_digest_module, gendigest
+from xpra.net.digest import get_digests, get_digest_module, gendigest
 
 
 def temp_filename(prefix=""):
@@ -33,7 +33,7 @@ class TempFileContext(object):
 			self.filename = self.file.name
 		return self
 
-	def __exit__(self, exc_type, exc_val, exc_tb):
+	def __exit__(self, _exc_type, _exc_val, _exc_tb):
 		if WIN32:
 			os.unlink(self.filename)
 
@@ -220,7 +220,7 @@ class TestAuth(unittest.TestCase):
 							assert not a.authenticate(verify, client_salt)
 
 	def test_file(self):
-		def genfiledata(a):
+		def genfiledata(_a):
 			password = uuid.uuid4().hex
 			return password, password
 		self._test_file_auth("file", genfiledata)
