@@ -21,8 +21,8 @@ from xpra.gtk_common.gobject_util import no_arg_signal, SIGNAL_RUN_LAST
 from xpra.gtk_common.gtk_util import GetClipboard, selection_owner_set, selection_add_target, selectiondata_get_selection, selectiondata_get_target, selectiondata_get_data, selectiondata_get_data_type, selectiondata_get_format, selectiondata_set, clipboard_request_contents, PROPERTY_CHANGE_MASK
 from xpra.gtk_common.nested_main import NestedMainLoop
 from xpra.net.compression import Compressible
-from xpra.os_util import WIN32, POSIX, monotonic_time, strtobytes, bytestostr, hexstr, get_hex_uuid
-from xpra.util import csv, envint, envbool, repr_ellipsized, typedict
+from xpra.os_util import WIN32, POSIX, monotonic_time, strtobytes, bytestostr, hexstr, get_hex_uuid, is_X11, is_Wayland
+from xpra.util import csv, envint, envbool, repr_ellipsized, typedict, first_time
 from xpra.platform.features import CLIPBOARD_GREEDY
 
 
@@ -66,9 +66,9 @@ TRANSLATED_TARGETS = {
     "application/x-moz-nativehtml" : "UTF8_STRING"
     }
 
-sizeof_long = struct.calcsize('@L')
+sizeof_long = struct.calcsize(b'@L')
 assert sizeof_long in (4, 8), "struct.calcsize('@L')=%s" % sizeof_long
-sizeof_short = struct.calcsize('=H')
+sizeof_short = struct.calcsize(b'=H')
 assert sizeof_short == 2, "struct.calcsize('=H')=%s" % sizeof_short
 
 
@@ -150,7 +150,7 @@ class ClipboardProtocolHelperBase(object):
 
     def get_info(self):
         info = {
-                "type"      : str(self),
+                "type"      : str(self).replace("ClipboardProtocolHelper", ""),
                 "max_size"  : self.max_clipboard_packet_size,
                 "filters"   : [x.pattern for x in self.filter_res],
                 "requests"  : self._clipboard_request_counter,
@@ -278,10 +278,16 @@ class ClipboardProtocolHelperBase(object):
         return  selection
 
     # Used by the client during startup:
+    def send_tokens(self, selections=[]):
+        for selection in selections:
+            proxy = self._clipboard_proxies.get(selection)
+            if proxy:
+                proxy._have_token = False
+                self._send_clipboard_token_handler(proxy)
+
     def send_all_tokens(self):
-        for proxy in self._clipboard_proxies.values():
-            proxy._have_token = False
-            self._send_clipboard_token_handler(proxy)
+        self.send_tokens(CLIPBOARDS)
+
 
     def _process_clipboard_token(self, packet):
         selection = bytestostr(packet[1])
@@ -429,11 +435,11 @@ class ClipboardProtocolHelperBase(object):
                 return None, None
             #important note: on 64 bits, format=32 means 8 bytes, not 4
             #that's just the way it is...
-            binfmt = "@" + "L" * (len(data) // sizeof_long)
+            binfmt = b"@" + b"L" * (len(data) // sizeof_long)
             ints = struct.unpack(binfmt, data)
             return b"integers", ints
         elif dformat == 16:
-            binfmt = "=" + "H" * (len(data) // sizeof_short)
+            binfmt = b"=" + b"H" * (len(data) // sizeof_short)
             ints = struct.unpack(binfmt, data)
             return b"integers", ints
         elif dformat == 8:
@@ -461,7 +467,7 @@ class ClipboardProtocolHelperBase(object):
                 format_char = "B"
             else:
                 raise Exception("unknown encoding format: %s" % dformat)
-            fstr = "@" + format_char * len(data)
+            fstr = b"@" + format_char * len(data)
             log("struct.pack(%s, %s)", fstr, data)
             return struct.pack(fstr, *data)
         else:
@@ -608,11 +614,12 @@ class ClipboardProxy(gtk.Invisible):
         self._request_contents_events = 0
         self._last_targets = ()
 
-        try:
-            from xpra.x11.gtk_x11.prop import prop_get
-            self.prop_get = prop_get
-        except ImportError:
-            self.prop_get = None
+        if is_X11():
+            try:
+                from xpra.x11.gtk_x11.prop import prop_get
+                self.prop_get = prop_get
+            except ImportError:
+                self.prop_get = None
 
         self._loop_uuid = ""
         self._clipboard.connect("owner-change", self.do_owner_changed)
@@ -778,6 +785,18 @@ class ClipboardProxy(gtk.Invisible):
         if not self._enabled or not self._can_receive:
             nodata()
             return
+        if not self._have_token:
+            try:
+                return gtk.Invisible.do_selection_get(self, selection_data, info, time)
+            except NotImplementedError as e:
+                log("gtk.Invisible.do_selection_get", exc_info=True)
+                if first_time("selection-%s-not-implemented" % self._selection):
+                    log.warn("Warning: limited clipboard support for %s", self._selection)
+                    if is_Wayland():
+                        log.warn(" looks like a Wayland implementation limitation")
+                    log.warn(" %s", e)
+                nodata()
+                return
         selection = selectiondata_get_selection(selection_data)
         target = selectiondata_get_target(selection_data)
         log("do_selection_get(%s, %s, %s) selection=%s", selection_data, info, time, selection)
@@ -877,14 +896,14 @@ class ClipboardProxy(gtk.Invisible):
     # This function is called by the xpra core when the peer has requested the
     # contents of this clipboard:
     def get_contents(self, target, cb):
-        log("get_contents(%s,%s) selection=%s, enabled=%s, can-send=%s", target, cb, self._selection, self._enabled, self._can_send)
+        log("get_contents(%s, %s) selection=%s, enabled=%s, can-send=%s", target, cb, self._selection, self._enabled, self._can_send)
         if not self._enabled or not self._can_send:
             cb(None, None, None)
             return
         self._get_contents_events += 1
         if self._have_token:
-            log.warn("Our peer requested the contents of the clipboard, but "
-                     + "*I* thought *they* had it... weird.")
+            log.warn("Warning: our peer requested the contents of the clipboard,")
+            log.warn(" but *I* thought *they* had it... weird.")
             cb(None, None, None)
             return
         if target=="TARGETS":
@@ -901,6 +920,7 @@ class ClipboardProxy(gtk.Invisible):
             log("unpack %s: %s", clipboard, type(selection_data))
             global sanitize_gtkselectiondata
             if selection_data and sanitize_gtkselectiondata(selection_data):
+                self._clipboard.set_text("", len=-1)
                 selection_data = None
             if selection_data is None:
                 cb(None, None, None)
